function [Adj, labels, x, y] = dot_to_graph(filename)
%%
%
% [Adj, labels, x, y] = dot_to_graph(filename)
% Extract an adjacency matrix, node labels from GraphViz file       http://www.research.att.com/sw/tools/graphviz
%
% INPUT:  'filename' - the file in DOT format containing the graph layout.
% OUTPUT: 'Adj' - an adjacency matrix with sequentially numbered edges; 
%     'labels'  - a character array with the names of the nodes of the graph;
%          


lines = textread(filename,'%s','delimiter','\n','commentstyle','c');  % Read file into cell array of lines
dot_lines = strvcat(lines);                                           % ignoring C-style comments

if isempty(findstr(dot_lines(1,:), 'graph '))   % Is this a DOT file ?
   error('* * * File does not appear to be in valid DOT format. * * *');
   return;
end;

Nlns = size(dot_lines,1);       % The number of lines;
labels = {};
unread = 1:Nlns;                % 'unread' list of lines which has not been examined yet
edge_id = 1;

for line_ndx = 1:Nlns 
    % This section sets the adjacency matrix entry A(Lnode,Rnode) =
    % edge_id. By default edje_id = 1
    line = dot_lines(line_ndx,:);
    
    % testing if arrow in the line of interest
    % if empty not a POS line
    arrow_pos = strfind(line, ' -> ');  % arrow  dash positions
    
    if not(isempty(arrow_pos))
    % parsing the line
    tokens = strread(line,'%s','delimiter','; "');
    Lnode = tokens(1);              % substrate
    Rnode = tokens(length(tokens)); % product
    
    
    % check if Left node already exists
    Lndx = strmatch(Lnode, labels, 'exact');
        
        if isempty(Lndx)         % extend our list of labels 
            labels(end+1) = Lnode;
            Lndx = length(labels);
        end
        
        % check if Right node already exists
        Rndx = strmatch(Rnode, labels, 'exact');
        
        if isempty(Rndx)
            labels(end+1) = Rnode;
            Rndx = length(labels);
        end
        
        Adj(Lndx, Rndx) = edge_id;
% checking if the matrix show same dimension than the labels vector length
        if ((length(labels)^2)> numel(Adj))
            Adj(Rndx,Rndx) = 0;
        end
        
    %edge_id = edge_id+1;
    end
end


%% extracting specific node names

for i = 1:Nlns
   
    line = dot_lines(i,:);
    
    % testing if bracket in the line of interest
    % if empty not a POS line
   namepos = [];
   name_pos = strfind(line, '[');  % bracket positions
   
   if not(isempty(name_pos))
       tokens = strread(line,'%s','delimiter','[=]  "');
       pos = strmatch(tokens(1),labels,'exact');
       labels(pos) = tokens(end-1);
   end
   
    
end